#include "controls.h"

void DoNothing() {}

Control::Control(HINSTANCE hInst, int ResIDNormal, int ResIDPressed, const Rect &rect) {

	pressed = false;
	
	pbmp = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(ResIDPressed), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
	ubmp = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(ResIDNormal), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
	PressedDC = CreateCompatibleDC(0);
	NormalDC = CreateCompatibleDC(0);
	SelectObject(PressedDC, pbmp);
	SelectObject(NormalDC, ubmp);

	ClickCallback = DoNothing;

	this->rect = rect;
}

Control::~Control() {
	DeleteObject(ubmp);
	DeleteObject(pbmp);
	DeleteDC(PressedDC);
	DeleteDC(NormalDC);
}

void Control::SetRect(int left, int top, int right, int bottom) {
	rect = Rect(left, top, right, bottom);
}

const Rect &Control::GetRect() const {
	return rect;
}

void Control::Pressed() {
	if(pressed) return;
	pressed = true;
}

void Control::Depressed() {
	if(!pressed) return;
	pressed = false;
	ClickCallback();
}

void Control::SetClickEvent(void (*callback)(void)) {
	ClickCallback = callback;
}

bool Control::HitTest(int x, int y) const {
	return x >= rect.left && x < rect.right && y >= rect.top && y < rect.bottom;
}

bool Control::IsPressed() const {
	return pressed;
}

//////// Button ///////////

Button::Button(HINSTANCE hInst, int ResIDNormal, int ResIDPressed, const Rect &rect) : Control(hInst, ResIDNormal, ResIDPressed, rect) {

}

void Button::Draw(HDC TargetDC) const {
	BitBlt(TargetDC, rect.left, rect.top, rect.right, rect.bottom, (pressed ? PressedDC : NormalDC), 0,0, SRCCOPY);
}

////////// SwitchButton //////////

SwitchButton::SwitchButton(HINSTANCE hInst, int ResIDNormal, int ResIDPressed, const Rect &rect) : Button(hInst, ResIDNormal, ResIDPressed, rect) {
	state = false;
}

void SwitchButton::Pressed() {
	pressed = !pressed;
	state = pressed;
}

void SwitchButton::Depressed() {
	if(!state) return;
	state = false;
	ClickCallback();
}



///////// ScrollBar /////////

ScrollBar::ScrollBar(HINSTANCE hInst, int ResIDNormal, int ResIDPressed, int ysize, const Rect &rect) : Control(hInst, ResIDNormal, ResIDPressed, rect) {
	BarRect = rect;
	ydiff = ysize;
	BarRect.bottom = BarRect.top + ydiff;
}

void ScrollBar::Draw(HDC TargetDC) const {
	BitBlt(TargetDC, BarRect.left, BarRect.top, BarRect.right, BarRect.bottom, (pressed ? PressedDC : NormalDC), 0,0, SRCCOPY);
}   

void ScrollBar::Pressed() {
	POINT pt;
	GetCursorPos(&pt);

	if(!pressed) {
		prevy = pt.y;
	} else {
		int offs = (pt.y - prevy);
		BarRect.top += offs;
		if(BarRect.top < rect.top) BarRect.top = rect.top;
		if(BarRect.top >= rect.bottom - ydiff) BarRect.top = rect.bottom - ydiff;
		BarRect.bottom = BarRect.top + ydiff;
		
		prevy = pt.y;
	}

	pressed = true;
}

float ScrollBar::GetPosition() const {
	return (float)(BarRect.top - rect.top) / (float)((rect.bottom - ydiff) - rect.top);
}

void ScrollBar::SetPosition(float newpos) {
	BarRect.top = rect.top + ((rect.bottom - ydiff) - rect.top) * newpos;
	if(BarRect.top < rect.top) BarRect.top = rect.top;
	if(BarRect.top >= rect.bottom - ydiff) BarRect.top = rect.bottom - ydiff;
	BarRect.bottom = BarRect.top + ydiff;
}

bool ScrollBar::HitTest(int x, int y) const {
	return x >= BarRect.left && x < BarRect.right && y >= BarRect.top && y < BarRect.bottom;
}
