#define WIN32_LEAN_AND_MEAN

#include <string>
#include <cassert>
#include <cstdio>
#include <windows.h>
#include "typedefs.h"
#include "color.h"
#include "timing.h"
#include "controls.h"
#include "linkedlist.h"
#include "nucwin/text.h"
#include "resource.h"

enum ProgramState {StateInit, StateMain, StateQuit};

int Init();
void Main();
void CleanUp();
void WindowMoved();
void LeftClick(int x, int y);
void LeftRelease(int x, int y);
void Drag(int x, int y);
void PaintWindow(dword t);
void EffectMainLoop();
LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

HWND hWnd;
HINSTANCE hInstance;
ProgramState State = StateInit;
bool Active = true;
const int WindowXSize = 326;
const int WindowYSize = 272;
Timer timer;

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR CmdLine, int ShowCmd) {

	MSG msg;
	hInstance = hInst;

	WNDCLASSEX wc;
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hbrBackground = (HBRUSH)GetStockObject(NULL_BRUSH);
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wc.hIconSm = wc.hIcon;
	wc.hInstance = hInst;
	wc.lpfnWndProc = WinProc;
	wc.lpszClassName = "NucWin";
	wc.lpszMenuName = NULL;
	wc.style = CS_HREDRAW | CS_VREDRAW;
	RegisterClassEx(&wc);

	hWnd = CreateWindowEx(0, "NucWin", "Vall@day Musicdisk", WS_POPUP | WS_VISIBLE, (GetSystemMetrics(SM_CXSCREEN)-WindowXSize)>>1, (GetSystemMetrics(SM_CYSCREEN)-WindowYSize)>>1, WindowXSize, WindowYSize, 0, 0, hInst, 0);

	if(Init()) State = StateMain;
	timer.Start();

	while(1) {
		if(PeekMessage(&msg, NULL, 0,0, PM_REMOVE)) {
			if(msg.message == WM_QUIT) break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		} else {
			if(State == StateMain) Main();
		}
	}
	return msg.wParam;
}

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {

	switch(msg) {
	case WM_CLOSE:
		State = StateQuit;
		CleanUp();
		DestroyWindow(hWnd);
		return 0;

	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;

	case WM_ACTIVATE:
		if(LOWORD(wParam) == WA_INACTIVE) Active = false; else Active = true;
		return 0;

	case WM_MOVE:
		WindowMoved();
		return 0;

	//case WM_PAINT:
	//	PaintWindow(timer.GetMilliSec());
	//	return 0;

	case WM_LBUTTONDOWN:
		LeftClick(LOWORD(lParam), HIWORD(lParam));
		return 0;

	case WM_LBUTTONUP:
		LeftRelease(LOWORD(lParam), HIWORD(lParam));
		return 0;

	case WM_MOUSEMOVE:
		if(wParam == MK_LBUTTON) {
			Drag(LOWORD(lParam), HIWORD(lParam));
			return 0;
		}
		break;


	default: break;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

///////////////////////////////////////////////////////////////////

void InitializeControls();

void OnInfo();
void OnExit();
void OnMinimize();
void OnUp();
void OnDown();
void OnPlay();
void OnStop();
void OnPause();
void OnRec();


HBITMAP WinDIB;
HDC BackDC;
byte *buffer;
HBITMAP Skin;
HDC SkinDC;

HBITMAP RotThingBmp[4];
HDC RotThing[4];

Button *BnQuit, *BnMin;
ScrollBar *SBarVolumeLevel, *SBarTracks;
SwitchButton *SBnPlay, *SBnPause;

LinkedList<Control*> controls;

NucText nuctext;

int Init() {

	BITMAPINFO binf;
	memset(&binf, 0, sizeof(BITMAPINFO));
	binf.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	binf.bmiHeader.biWidth = WindowXSize;
	binf.bmiHeader.biHeight = WindowYSize;
	binf.bmiHeader.biBitCount = 24;
	binf.bmiHeader.biPlanes = 1;
	binf.bmiHeader.biCompression = BI_RGB;
	
	HDC hdc = GetDC(hWnd);
	WinDIB = CreateDIBSection(hdc, &binf, DIB_RGB_COLORS, (void**)&buffer, 0, 0);
	ReleaseDC(hWnd, hdc);
	BackDC = CreateCompatibleDC(0);
	SelectObject(BackDC, WinDIB);

	Skin = LoadBitmap(hInstance, MAKEINTRESOURCE(IDB_INTERFACE));
	SkinDC = CreateCompatibleDC(0);
	SelectObject(SkinDC, Skin);

	for(int i=0; i<4; i++) {
		RotThingBmp[i] = LoadBitmap(hInstance, MAKEINTRESOURCE(IDB_ROTBAR1+i));
		RotThing[i] = CreateCompatibleDC(0);
		SelectObject(RotThing[i], RotThingBmp[i]);
	}
	
	// Set window regions
	HRSRC hrSkin = FindResource(hInstance, MAKEINTRESOURCE(IDR_REGIONS),"BINARY");
	RGNDATA *SkinData = (RGNDATA*)LoadResource(hInstance, hrSkin);
	HRGN Region = ExtCreateRegion(0, SizeofResource(0, hrSkin), SkinData);
	FreeResource(SkinData);
	SetWindowRgn(hWnd, Region, true);
	DeleteObject(Region);

	InitializeControls();


	return 1;
}

void Main() {
	
	PaintWindow(timer.GetMilliSec());
	EffectMainLoop();
	Sleep(20);
}

void CleanUp() {

	DeleteDC(SkinDC);
	DeleteDC(BackDC);
	DeleteObject(WinDIB);
	DeleteObject(Skin);	

	for(int i=0; i<4; i++) {
		DeleteObject(RotThingBmp[i]);
		DeleteDC(RotThing[i]);
	}

}

void WindowMoved() {

}

void InitializeControls() {
	Button *but = new Button(hInstance, IDB_INFO, IDB_INFOP, Rect(187, 202, 201, 211));
	but->SetClickEvent(OnInfo);
	controls.PushBack(but);

	but = new Button(hInstance, IDB_EXIT, IDB_EXITP, Rect(185, 23, 199, 38));
	but->SetClickEvent(OnExit);
	controls.PushBack(but);

	but = new Button(hInstance, IDB_MIN, IDB_MINP, Rect(172, 23, 185, 38));
	but->SetClickEvent(OnMinimize);
	controls.PushBack(but);

	SBnPause = new SwitchButton(hInstance, IDB_PAUSE, IDB_PAUSEP, Rect(173, 56, 184, 67));
	SBnPause->SetClickEvent(OnPause);
	controls.PushBack(SBnPause);

	SBnPlay = new SwitchButton(hInstance, IDB_PLAY, IDB_PLAYP, Rect(151, 56, 162, 67));
	SBnPlay->SetClickEvent(OnPlay);
	controls.PushBack(SBnPlay);

	but = new Button(hInstance, IDB_STOP, IDB_STOPP, Rect(162, 56, 173, 67));
	but->SetClickEvent(OnStop);
	controls.PushBack(but);

	but = new Button(hInstance, IDB_REC, IDB_RECP, Rect(198, 56, 209, 67));
//	but->SetClickEvent(OnRec);
	controls.PushBack(but);

	but = new Button(hInstance, IDB_SCRLDOWN, IDB_SCRLDOWNP, Rect(207, 182, 214, 189));
	but->SetClickEvent(OnDown);
	controls.PushBack(but);

	but = new Button(hInstance, IDB_SCRLUP, IDB_SCRLUPP, Rect(207, 71, 214, 78));
	but->SetClickEvent(OnUp);
	controls.PushBack(but);

	SBarTracks = new ScrollBar(hInstance, IDB_SCROLLBAR, IDB_SCROLLBARP, 24, Rect(207, 79, 214, 181));
	// set events
	controls.PushBack(SBarTracks);

	SBarVolumeLevel = new ScrollBar(hInstance, IDB_VOLUMEBAR, IDB_VOLUMEBAR, 8, Rect(35, 77, 44, 146));
	// events
	controls.PushBack(SBarVolumeLevel);
}



void PaintWindow(dword t) {

	PAINTSTRUCT ps;
	BeginPaint(hWnd, &ps);

	BitBlt(BackDC, 0, 0, WindowXSize, WindowYSize, SkinDC, 0,0, SRCCOPY);	
	BitBlt(BackDC, 219, 101, 226, 162, RotThing[(t/100)%4], 0,0, SRCCOPY);

	char vol[25];
	sprintf(vol, "Volume: %d", (int)((1.0f - SBarVolumeLevel->GetPosition()) * 100.0f));
	SetBkMode(BackDC, TRANSPARENT);
	nuctext.SetFont("Verdana", 7, 0x0dcc0d);
	nuctext.Print(vol, BackDC, 47, 196);

	ListNode<Control*> *iter = controls.Begin();
	while(iter) {
		iter->data->Draw(BackDC);
		iter = iter->next;
	}

	HDC hdc = GetDC(hWnd);
	BitBlt(hdc, 0, 0, WindowXSize, WindowYSize, BackDC, 0,0, SRCCOPY);
	ReleaseDC(hWnd, hdc);

	EndPaint(hWnd, &ps);
}

void LeftClick(int x, int y) {
	ListNode<Control*> *iter = controls.Begin();
	while(iter) {
		if(iter->data->HitTest(x, y)) {
			iter->data->Pressed();
			return;
		}
		iter = iter->next;
	}
	
	SendMessage(hWnd, WM_NCLBUTTONDOWN, HTCAPTION, 0);
}

void LeftRelease(int x, int y) {
	ListNode<Control*> *iter = controls.Begin();
	while(iter) {
		iter->data->Depressed();
		iter = iter->next;
	}
}

void Drag(int x, int y) {
	ListNode<Control*> *iter = controls.Begin();
	while(iter) {
		if(iter->data->IsPressed()) {
			iter->data->Pressed();
			return;
		}
		iter = iter->next;
	}
}

/////////// Event handlers /////////////

void OnInfo() {
	MessageBox(0, "Information shit", "about vall@day music disk", MB_OK);
	return;
}

void OnExit() {
	SendMessage(hWnd, WM_CLOSE, 0, 0);
}

void OnMinimize() {
	CloseWindow(hWnd);
}

void OnUp() {
	SBarTracks->SetPosition(SBarTracks->GetPosition() - 0.05);
}

void OnDown() {
	SBarTracks->SetPosition(SBarTracks->GetPosition() + 0.05);
}

void OnPlay() {
	//if(SBnPlay->IsPressed()) SBnPlay->Pressed();
	if(SBnPause->IsPressed()) SBnPause->Pressed();
}

void OnPause() {
	if(!SBnPlay->IsPressed()) SBnPause->Pressed();
}

void OnStop() {
	if(SBnPlay->IsPressed()) SBnPlay->Pressed();
	if(SBnPause->IsPressed()) SBnPause->Pressed();
}

void OnRec() {}

////////////////////////

void EffectMainLoop() {
	float t = (float)timer.GetMilliSec() / 1000.0f;
}


